// Copyright 2009-present MongoDB, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include <bsoncxx/json.hpp>

#include <mongocxx/client.hpp>
#include <mongocxx/options/client.hpp>
#include <mongocxx/options/server_api.hpp>
#include <mongocxx/uri.hpp>

#include <examples/api/runner.hh>
#include <examples/macros.hh>

namespace {

// [Example]
mongocxx::client example() {
    mongocxx::options::server_api server_api_opts{
        mongocxx::options::server_api::version::k_version_1};

    server_api_opts.strict(true);
    // ... other Stable API options.

    mongocxx::options::client client_opts;

    client_opts.server_api_opts(server_api_opts);

    mongocxx::uri uri;
    mongocxx::client client{uri, client_opts};

    EXPECT(client);
    EXPECT(client.uri().to_string() == mongocxx::uri::k_default_uri);

    return client;
}
// [Example]

}  // namespace

RUNNER_REGISTER_COMPONENT_FOR_SINGLE() {
    auto client = example();

    auto reply = client["admin"].run_command(bsoncxx::from_json(R"({"ping": 1})"));

    EXPECT(reply["ok"] && reply["ok"].get_double().value == 1.0);
}
